<?php
/*
Plugin Name: Eventbrite Events - Public API Version
Plugin URI: https://www.eventbrite.com
Description: Provides the custom meta to define which pages are for which
purpose and renders them. Support is enabled for venue and event details, and
list view and search
Contributors: Marco Schmoecker, Yoli Hodde & Matt Harris
Version: 2.0.0
Author URI: https://www.eventbrite.com
*/

define( 'EB_VERSION', '2' );
define( 'EVB_API_ENDPOINT', 'https://www.eventbriteapi.com' );
define( 'IMGIX_SIG', '3h77vPcS' );

require_once dirname( __FILE__ ) . '/includes/api/Oauth.php';
require_once dirname( __FILE__ ) . '/includes/api/Performer_Api.php';
require_once dirname( __FILE__ ) . '/includes/api/Events_Api.php';
require_once dirname( __FILE__ ) . '/includes/api/Event_Api.php';
require_once dirname( __FILE__ ) . '/includes/api/Description_Api.php';
require_once dirname( __FILE__ ) . '/includes/tf_functions.php';
require_once dirname( __FILE__ ) . '/includes/eb_functions.php';
require_once dirname( __FILE__ ) . '/includes/venueInfo.php';
require_once dirname( __FILE__ ) . '/includes/performer_class.php';

if ( ! class_exists( 'eventbrite_music' ) ) :

	class eventbrite_music {
		/**
		 * @var Oauth
		 */
		protected $oauth;

		protected $id = 'eventbrite_music';

		protected $fields = [
			'eb_events_page_role' => "\n",
		];

		protected $roles = [
			''              => 'None',
			'event_details' => 'Event Details',
			'calendar'      => 'Calendar',
			'listing'       => 'Listing',
		];

		/**
		 * Array of supported setting options
		 *
		 * @var array of option keys and display values
		 */
		protected function plugin_options() {
			return [
				'radio_filter_endpoint' => [
					'name'         => 'Data Source',
					'option_group' => 'set',
					'options'      => [ 'Organizers', 'Venues' ],
					'info'         => '',
				],
				'org_id'                => [
					'name'         => 'Organizations',
					'option_group' => 'set',
					'type'         => 'multiselect',
					'options'      => $this->oauth->get_user_organizations(),
				],
				'venue_id'              => [
					'name'         => 'Venues',
					'option_group' => 'set',
					'type'         => 'multiselect',
					'options'      => $this->oauth->get_venues_for_organizations(),
				],
				'api_key'               => [
					'name'         => 'API Key',
					'option_group' => 'oauth',
					'type'         => 'password',
				],
				'client_secret'         => [
					'name'         => 'Client Secret',
					'option_group' => 'oauth',
					'type'         => 'password',
				],
				'backup_site_url'       => [
					'name'         => 'Backup Site URL',
					'option_group' => 'cust',
					'info'         => 'If there is a problem with the API, a link to the backup site will be displayed. Ex. http://www.yourbackupsite.com',
				],
				'enable_css'            => [
					'name'         => 'Enable CSS',
					'option_group' => 'cust',
					'info'         => 'If you wish to not load the Eventbrite Events Plugin CSS file and use your own local CSS definitions, uncheck the box.',
				],
				'enable_venue_name'     => [
					'name'         => 'Output Venue Name',
					'option_group' => 'cust',
					'info'         => 'Toggles the display of the name, city, and state of the venue in the List View. Promoters who have events at multiple venues should check the box.',
				],
				'enable_purchase_modal' => [
					'name'         => 'Embedded Modal',
					'option_group' => 'cust',
					'info'         => 'Use Eventbrite Embedded Checkout modal.',
				],
				'enable_purchase_links' => [
					'name'         => 'Purchase Links',
					'option_group' => 'cust',
					'info'         => 'Open purchase page in a new browser tab.',
				],
				'ga_enable_ganalytics'  => [
					'name'         => 'Google Analytics',
					'option_group' => 'ga',
					'info'         => 'Do you want to track your site with Google Analytics?',
					'extrainfo'    => 'Please note that by disabling this you will no longer see Source Performance.',
				],
				'ga_ua_code'            => [ 'name' => 'Google Analytics UA Code', 'option_group' => 'ga' ],
				'affiliate_code'        => [ 'name' => 'Affiliate Code', 'option_group' => 'ga' ],
			];
		}


		/**
		 * @var string Version of the generator in SVN
		 */
		private static $pluginVersion = '2.0';

		/**
		 * The key to use for the options and all input/id prefixes
		 *
		 * @var string the site option key
		 */
		protected $settings_ref = 'eventbrite_plugin';

		function __construct() {
			$this->oauth = new Oauth();
			$this->oauth->hooks();

			$this->defaults = [
				'radio_filter_endpoint' => 'organizer_ids',
				'org_id'                => [],
				'venue_id'              => [],
				'client_secret'         => '',
				'api_key'               => '',
				'enable_css'            => '1',
				'enable_venue_name'     => 0,
				'enable_purchase_modal' => '1',
				'enable_purchase_links' => 0,
				'ga_enable_ganalytics'  => 0,
				'ga_ua_code'            => '',
				'affiliate_code'        => 'odwdwdclientsite',
			];

			add_action( 'admin_init', [ $this, 'admin_init' ] );
			add_action( 'admin_menu', [ $this, 'admin_menu' ] );

			add_action( 'admin_head-post.php', [ $this, 'admin_head' ] );
			add_action( 'admin_head-post-new.php', [ $this, 'admin_head' ] );
			add_action( 'parse_request', [ $this, 'eb_parse_request' ] );
			add_action( 'save_post', [ $this, 'save_post' ], 10, 2 );
			add_action( 'save_post', [ $this, 'update_pagerole_url' ] );
			add_filter( 'posts_results', [ $this, 'posts_results' ] );
			add_action( 'wp_enqueue_scripts', [ $this, 'add_eb_stylesheet' ] );
			add_action( 'wp_enqueue_scripts', [ $this, 'eb_shortcodeEnqueue' ] );

			// we are using the filter here as a hook to run an API query
			if ( ! is_admin() ) {
				add_filter( 'the_posts', [ $this, 'the_posts' ] );
				add_filter( 'single_post_title', [ $this, 'eb_title' ], 10, 2 );
				add_action( 'get_header', [ $this, 'eb_removeAddOGMeta' ], 10, 2 );
			}

			add_action( 'wp_head', [ $this, 'eb_fb_share_image' ], 2 );
			add_action( 'wp_head', [ $this, 'eb_printGTM' ], 1 );
			add_action( 'wp_head', [ $this, 'eb_printGaTracking' ], 1 );
			add_action( 'sidebar_admin_page', [ $this, 'eb_widgets_options_filter' ] );

			add_filter( 'rewrite_rules_array', [ $this, 'rewrite_rules_array' ] );
			add_filter( 'query_vars', [ $this, 'query_vars' ] );
			add_action( 'init', [ $this, 'init' ] );

			add_shortcode( 'tf_calendar', [ $this, 'eb_cal_func' ] );
			add_shortcode( 'eb_calendar', [ $this, 'eb_cal_func' ] );
			add_shortcode( 'tf_event_details', [ $this, 'eb_details_func' ] );
			add_shortcode( 'eb_event_details', [ $this, 'eb_details_func' ] );
			add_shortcode( 'tf_carousel', [ $this, 'eb_carousel_func' ] );
			add_shortcode( 'eb_carousel', [ $this, 'eb_carousel_func' ] );
			add_shortcode( 'tf_upcoming', [ $this, 'eb_upcoming_func' ] );
			add_shortcode( 'eb_upcoming', [ $this, 'eb_upcoming_func' ] );
			add_shortcode( 'tf_hero_listview', [ $this, 'eb_heroListView_func' ] );
			add_shortcode( 'eb_hero_listview', [ $this, 'eb_heroListView_func' ] );
			add_shortcode( 'tf_listview', [ $this, 'eb_listView_func' ] );
			add_shortcode( 'eb_listview', [ $this, 'eb_listView_func' ] );
			add_shortcode( 'tf_searchform', [ $this, 'eb_searchForm_func' ] );
			add_shortcode( 'eb_searchform', [ $this, 'eb_searchForm_func' ] );
			add_shortcode( 'tf_powered_link', [ $this, 'eb_powered_func' ] );
			add_shortcode( 'eb_powered_link', [ $this, 'eb_powered_func' ] );
			add_shortcode( 'tf_emailform', [ $this, 'eb_emailForm_func' ] );
			add_shortcode( 'eb_emailform', [ $this, 'eb_emailForm_func' ] );
			add_shortcode( 'tf_justannounced', [ $this, 'eb_justAnnounced_func' ] );
			add_shortcode( 'eb_justannounced', [ $this, 'eb_justAnnounced_func' ] );
			add_shortcode( 'tf_onsale', [ $this, 'tf_onSale_func' ] );
			add_shortcode( 'tf_attraction_details', [ $this, 'tf_deprecated_func' ] );
			add_shortcode( 'tf_featuredpromoters', [ $this, 'tf_deprecated_func' ] );
			add_shortcode( 'tf_venueinfo', [ $this, 'tf_deprecated_func' ] );
			add_shortcode( 'tf_youtube_player', [ $this, 'tf_deprecated_func' ] );
			add_shortcode( 'tf_venuelist', [ $this, 'tf_deprecated_func' ] );
			add_filter( 'widget_text', 'do_shortcode' );
		}

		/**
		 * Shortcode: Ticketfly shortcode that will not be supported in EB
		 *
		 * @return void
		 */
		function tf_deprecated_func() {
			echo '';
		}

		/**
		 * Returns XML name space for FB API.
		 *
		 * @return string
		 * @author Marco Schmoecker
		 **/
		function eb_addFBClasses( $output ) {
			return 'xmlns:og="http://opengraphprotocol.org/schema/" xmlns:fb="https://www.facebook.com/2008/fbml" ' . $output;
		}

		/**
		 * Make sure to remove FB plugin OG meta and add in our own
		 *
		 * @author Marco Schmoecker
		 **/
		function eb_removeAddOGMeta() {
			if ( is_page( [ 'e', 'org', 'venue' ] ) ) {
				add_action( 'wp_head', [ $this, 'eb_addCanonicalMeta' ], 2 );
				if ( is_page( 'e' ) ) {
					add_action( 'wp_head', [ $this, 'eb_addOpenGraphMeta' ] );
				}
			}
		}


		/**
		 * Add canonical meta tag for event detail pages
		 *
		 * -> inject meta tag into page header
		 *
		 * @return void
		 * @author Marco Schmoecker
		 **/
		function eb_addCanonicalMeta() {
			if ( ! is_admin() ) {
				// Inject EB canonical meta tag.
				global $post;

				$canonicalPostUrl = '';

				if ( is_page( 'e' ) && isset( $post->eb_event->id ) ) {
					$canonicalPostUrl = eb_URL( 'eb_event_details', $post->eb_event->id, '',
						[ 'eventSlug' => sanitize_title_with_dashes( $post->eb_event->name->text ) ] );
				} elseif ( isset( $post->eb_events_page_role ) && ( 'attraction' === $post->eb_events_page_role ) ) {
					$canonicalPostUrl = eb_URL( 'attraction', $post->eb_id );
				} elseif ( isset( $post->eb_type ) && isset( $post->tfly_orgVenueName ) && ( ( 'venue' === $post->eb_type ) || ( 'org' === $post->eb_type ) ) ) {
					if ( 'org' === $post->eb_type ) {
						$canonicalPostUrl = eb_URL( 'org', $post->eb_id, '',
							[ 'orgVenueName' => $post->tfly_orgVenueName ] );
					}
					if ( 'venue' === $post->eb_type ) {
						$canonicalPostUrl = eb_URL( 'venue', $post->eb_id, '',
							[ 'orgVenueName' => $post->tfly_orgVenueName ] );
					}
				}

				if ( ! empty( $canonicalPostUrl ) ) {
					echo '<link rel="canonical" href="' . esc_url( $canonicalPostUrl ) . '" />';
				}
			}
		}

		/**
		 * Add OpenGraph meta tags for event detail pages (event data)
		 *
		 * -> inject meta tags into page header
		 *
		 * @author Marco Schmoecker
		 **/
		function eb_addOpenGraphMeta() {
			if ( is_admin() ) {
				return;
			}

			global $post;
			$request_uri = isset( $_SERVER[ 'REQUEST_URI' ] ) ? sanitize_text_field( $_SERVER[ 'REQUEST_URI' ] ) : '';
			$postUrl     = site_url( wp_parse_url( $request_uri, PHP_URL_PATH ) );
			$postTitle   = $post->post_title;
			if ( is_front_page() ) {
				$postTitle = get_bloginfo( 'name' );
			}

			$siteName = get_bloginfo( 'name' );

			$eventData = '';
			if ( isset( $post->eb_event ) ) {
				$eventData = $post->eb_event;
			}

			echo '<meta name="twitter:card" content="summary">';
			$twitterHandle = '@eventbrite';
			if ( isset( $eventData->primary_organizer->twitter ) && $eventData->primary_organizer->twitter ) {
				$twitterHandle = '@' . $eventData->primary_organizer->twitter;
			}
			echo '<meta name="twitter:site" content="' . esc_attr( $twitterHandle ) . '">';

			// todo: add in proper twitter handle, once we have artist data
			if ( isset( $eventData->headliners[ 0 ]->twitter ) ) {
				$twitterHandle = $eventData->headliners[ 0 ]->twitter;
			}
			echo '<meta name="twitter:creator" content="' . esc_attr( $twitterHandle ) . '">';

			$siteDescription = ( $eventData->summary ) ? substr( ltrim( str_replace( [
				"\n",
				"\r",
			], ' ', _wp_specialchars( wp_strip_all_tags( $eventData->summary ), ENT_COMPAT ) ) ), 0,
				290 ) : $postTitle;

			echo '<meta property="og:title" content="' . esc_attr( $postTitle ) . '" />';
			echo '<meta property="og:description" content="' . esc_attr( $siteDescription ) . '" />';
			echo '<meta property="og:site_name" content="' . esc_attr( $siteName ) . '" />';
			echo '<meta property="og:url" content="' . esc_attr( $postUrl ) . '" />';
			echo '<meta property="og:type" content="eventbrite:event" />';

			if ( isset( $eventData->image->original->url ) ) {
				$tfly_share_image = $eventData->image->original->url;
			} else {
				$tfly_share_image = eb_plugin_dir_URL() . '/images/no-image.png';
			}

			echo '<meta property="og:image" content="' . esc_url( $tfly_share_image ) . '" />';
			if ( isset( $eventData->image->original->width ) && $eventData->image->original->width ) {
				echo '<meta property="og:image:width" content="' . absint( $eventData->image->original->width ) . '" />';
			}
			if ( isset( $eventData->image->original->height ) && $eventData->image->original->height ) {
				echo '<meta property="og:image:height" content="' . absint( $eventData->image->original->height ) . '" />';
			}

			if ( $eventData->venue ) {
				if ( isset( $eventData->venue->address->postal_code ) ) {
					echo '<meta property="og:postal-code" content="' . esc_attr( $eventData->venue->address->postal_code ) . '" />';
				}
				if ( isset( $eventData->venue->address->region ) ) {
					echo '<meta property="og:country-name" content="' . esc_attr( $eventData->venue->address->region ) . '" />';
				}
				if ( isset( $eventData->venue->address->city ) ) {
					echo '<meta property="og:locality" content="' . esc_attr( $eventData->venue->address->city ) . '" />';
				}
				if ( isset( $eventData->venue->address->address_1 ) || isset( $eventData->venue->address->address_2 ) ) {
					$addressArray = [];
					if ( ! empty( $eventData->venue->address->address_1 ) ) {
						$addressArray[] = $eventData->venue->address->address_1;
					}
					if ( ! empty( $eventData->venue->address->address_2 ) ) {
						$addressArray[] = $eventData->venue->address->address_2;
					}
					$address = implode( ', ', $addressArray );
					echo '<meta property="og:street-address" content="' . esc_attr( $address ) . '" />';
				}
				if ( isset( $eventData->venue->address->region ) ) {
					echo '<meta property="og:region" content="' . esc_attr( $eventData->venue->address->region ) . '" />';
				}
				if ( isset( $eventData->venue->address->latitude ) ) {
					echo '<meta property="og:latitude" content="' . esc_attr( $eventData->venue->address->latitude ) . '" />';
				}
				if ( isset( $eventData->venue->address->longitude ) ) {
					echo '<meta property="og:lng" content="' . esc_attr( $eventData->venue->address->longitude ) . '" />';
				}
			}
		}

		/**
		 * Enqueues assets required by shortcodes only when shortcodes are active
		 *
		 * @return void
		 **/
		function eb_shortcodeEnqueue() {
			wp_register_script( 'owl-carousel-js', plugins_url( '/js/owl.carousel.min.js', __FILE__ ), [ 'jquery' ],
				'2.3.4', true );
			wp_register_script( 'owl-carousel-aria', plugins_url( '/js/owl.carousel.aria.min.js', __FILE__ ),
				[ 'jquery' ], EB_VERSION, true );
			wp_register_style( 'owl-carousel', plugins_url( '/css/owl.carousel.min.css', __FILE__ ), [], '2.3.4',
				'all' );
			wp_register_style( 'owl-theme', plugins_url( '/css/owl.theme.default.min.css', __FILE__ ), [], '2.3.4',
				'all' );
		}

		/**
		 * Enqueue style-file, if it exists.
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function add_eb_stylesheet() {
			$ebStyleFile = plugin_dir_path( __FILE__ ) . 'style.css';
			$options     = $this->get_options();
			if ( file_exists( $ebStyleFile ) && $options[ 'enable_css' ] ) {
				$ebStyleUrl = eb_plugin_dir_URL() . '/style.css';
				wp_register_style( 'ebStyleSheets', $ebStyleUrl, false, EB_VERSION );
				wp_enqueue_style( 'ebStyleSheets' );
			}

			$ebStyleFile = plugin_dir_path( __FILE__ ) . 'css/eb-hide.css';
			if ( file_exists( $ebStyleFile ) ) {
				$ebHideStyleUrl = eb_plugin_dir_URL() . '/css/eb-hide.css';
				wp_register_style( 'ebHideStyleSheets', $ebHideStyleUrl, false, EB_VERSION );
				wp_enqueue_style( 'ebHideStyleSheets' );
			}
		}

		/**
		 * Add short code help below the widget areas.
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function eb_widgets_options_filter() {
			?>
			<div class="wrap">
				<h2>Eventbrite Short Code Help</h2>
				<p>
				<ul style="list-style-type:square;list-style-position:inside;">
					<li>[eb_calendar]</li>
					<li>[eb_carousel]</li>
					<li>[eb_event_details]</li>
					<li>[eb_justannounced]</li>
					<li>[eb_listview]</li>
					<li>[eb_powered_link]</li>
					<li>[eb_searchform]</li>
					<li>[eb_upcoming]</li>
				</ul>
				</p>
				<p>
					Detailed instructions &amp;amp; options on the <a
							href="<?php echo esc_url( admin_url( 'admin.php?page=eb-events.php-help' ) ); ?>">Help
						page</a>
				</p>
			</div>
			<?php
		}


		/**
		 * Verify we're a site admin and register the setting options.
		 *
		 * @return void
		 * @contributors Matt Harris, Marco Schmoecker
		 */
		function admin_init() {
			register_setting( $this->settings_ref, $this->settings_ref, [ $this, 'options_validate' ] );
			add_settings_section( 'config_oauth', __( 'Oauth Settings', 'eventbrite' ),
				[ $this, 'render_section_text' ], "{$this->settings_ref}_page" );
			add_settings_section( 'config_set', __( 'Event Data Settings', 'eventbrite' ),
				[ $this, 'render_section_text' ],
				"{$this->settings_ref}_page" );
			add_settings_section( 'config_cust', __( 'Customization', 'eventbrite' ),
				[ $this, 'render_section_text' ], "{$this->settings_ref}_page" );
			add_settings_section( 'config_ga', __( 'Google Analytics', 'eventbrite' ),
				[ $this, 'render_section_text' ], "{$this->settings_ref}_page" );

			foreach ( $this->plugin_options() as $key => $value ) {
				add_settings_field(
					$key,
					__( $value[ 'name' ], 'eventbrite' ),
					[ $this, 'render' ],
					"{$this->settings_ref}_page",
					'config_' . $value[ 'option_group' ],
					$key
				);
			}
		}

		/**
		 * Outputs the text to display in the header of the options section
		 *
		 * @return void
		 * @authors Matt Harris, some mods by Marco Schmoecker
		 */
		function render_section_text( $data ) {
			switch ( $data[ 'id' ] ) {
				case 'config_set':
					echo '<p>' . esc_html__( 'Please configure site options in the area below. Choose data source first and then choose which organizations or venues to pull events from. (Note: Events can only be pulled in from either organziations or venues, but not both.) To choose multiple organizations or venues, hold down the Ctrl (PC) or Command (Mac) key and click on your desired items.',
							'eventbrite' ) . '</p>';
					break;
				case 'config_oauth':
					echo sprintf( __( '<p>Visit <a href="https://www.eventbrite.com/platform/api#/introduction/authentication/2.-(for-app-partners)-authorize-your-users" target="_blank">https://www.eventbrite.com/platform/api#/introduction/authentication/2.-(for-app-partners)-authorize-your-users</a> for information on getting your API Key. Your "Oauth Redirect URI" will be <code>%s</code></p>',
						'eventbrite' ), esc_url( admin_url( 'admin.php?page=' . basename( __FILE__ ) ) ) );
					break;
				case 'config_cust':
					echo '<p>' . esc_html__( 'Please configure the various customizations in the area below.',
							'eventbrite' ) . '</p>';
					break;
				case 'config_ga':
					echo '<p>' . esc_html__( 'Please configure Google Analytics in the area below.',
							'eventbrite' ) . '</p>';
					break;
			}
		}

		/**
		 * Renders out option field
		 *
		 * @param string $what the option to render
		 *
		 * @return void
		 * @contributors Matt Harris, Marco Schmoecker
		 */
		function render( $what ) {
			$options        = $this->get_options();
			$plugin_options = $this->plugin_options();
			if ( strstr( $what, 'enable' ) ) {
				$checked       = ( ( $options[ $what ][ 0 ] === '1' ) || $options[ $what ] === '1' ) ? 'checked=checked' : '';
				$safe_xtrainfo = isset( $plugin_options[ $what ][ 'extrainfo' ] )
					? '<br /><span style="font-size:0.9em;font-style:italic;padding-left:32px;">' . esc_html( $plugin_options[ $what ][ 'extrainfo' ] ) . '</span>'
					: '';
				echo sprintf( '<input name="%s[%s]" type="hidden" value="0" />',
					esc_attr( $this->settings_ref ),
					esc_attr( $what )
				);

				echo sprintf( '<input id="%1$s_%2$s" name="%1$s[%2$s]" type="checkbox" value="1" %3$s>',
					esc_attr( $this->settings_ref ),
					esc_attr( $what ),
					esc_html( $checked )
				);

				echo esc_html( $plugin_options[ $what ][ 'info' ] ) . $safe_xtrainfo;
			} elseif ( strstr( $what, 'radio' ) ) {
				$newLine = false;
				foreach ( $plugin_options[ $what ][ 'options' ] as $o ) {
					$checked = ( $options[ $what ] === strtolower( $o ) ) ? 'checked=checked' : '';
					if ( $newLine ) {
						echo '<br/>';
					}
					$newLine = true;
					echo sprintf( '<input id="%1$s_%2$s" name="%1$s[%2$s]" type="radio" value="%3$s" %4$s />  %5$s',
						esc_attr( $this->settings_ref ),
						esc_attr( $what ),
						esc_attr( strtolower( $o ) ),
						esc_attr( $checked ),
						esc_html( $o )
					);
				}
			} elseif ( 'org_id' === $what ) {
				if ( empty( $plugin_options[ $what ][ 'options' ] ) ) {
					echo '<em>' . __( 'Setup your Oauth Settings to select organizations.',
							'eventbrite' ) . '</em>';
				} else {
					echo sprintf( '<div><select id="%1$s_%2$s" name="%1$s[%2$s][]" multiple="multiple">%3$s</select></div>',
						esc_attr( $this->settings_ref ),
						esc_attr( $what ),
						$this->get_select_options( $plugin_options[ $what ][ 'options' ], $options[ $what ] )
					);
				}
			} elseif ( 'venue_id' === $what ) {
				if ( empty( $plugin_options[ $what ][ 'options' ] ) ) {
					echo '<em>' . __( 'Setup your Oauth Settings to select venues.',
							'eventbrite' ) . '</em>';
				} else {
					echo sprintf( '<div><select id="%1$s_%2$s" name="%1$s[%2$s][]" multiple="multiple">%3$s</select></div>',
						esc_attr( $this->settings_ref ),
						esc_attr( $what ),
						$this->get_select_options( $plugin_options[ $what ][ 'options' ], $options[ $what ] )
					);
				}
			} else {
				$type = isset( $plugin_options[ $what ][ 'type' ] ) ? $plugin_options[ $what ][ 'type' ] : 'text';
				echo sprintf( '<input id="%1$s_%2$s" name="%1$s[%2$s]" size="44" type="%4$s" value="%3$s">',
					esc_attr( $this->settings_ref ),
					esc_attr( $what ),
					isset( $options[ $what ] ) ? esc_attr( $options[ $what ] ) : '',
					esc_attr( $type )
				);
				if ( 'venue_id' === $what ) { ?>
					<input style="margin-left:35px;" type="submit" name="submit" class="button-primary"
						   value="<?php esc_html_e( 'Update', 'eventbrite' ); ?>">
					<?php
				}
				if ( 'client_secret' === $what && ! empty( $options[ 'api_key' ] ) ) {
					echo '<p style="padding-bottom: 10px">' . __( 'If you change the API Key value you must save the settings before generating a new token',
							'eventbrite' ) . '</p>';
					$auth_url = 'https://www.eventbrite.com/oauth/authorize?response_type=code&client_id=' . $options[ 'api_key' ] . '&redirect_uri=' . esc_url( admin_url( 'admin.php?page=' . basename( __FILE__ ) ) ); ?>
					<a href="<?php echo esc_url( $auth_url ); ?>" class="button">
						<?php echo __( 'Generate Auth Token API', 'eventbrite' ); ?>
					</a>
					<?php
				}
				if ( isset( $plugin_options[ $what ][ 'info' ] ) ) { ?>
					<p style="font-size:0.9em;font-style:italic;"><?php echo wp_kses_post( $plugin_options[ $what ][ 'info' ] ) ?></p>
				<?php }
			}
		}

		public function get_select_options( $choices, $selected ) {
			if ( empty( $choices ) ) {
				return '';
			}
			$options = '';
			foreach ( $choices as $id => $name ) {
				$options .= sprintf( '<option value="%1$s" %2$s>%3$s</option>',
					$id,
					in_array( $id, $selected ) ? 'selected' : '',
					$name
				);
			}

			return $options;
		}

		/**
		 * Verify we're a site admin and register the admin menu entries.
		 * Also register the load hook so our options get saved on post back.
		 *
		 * @return void
		 * @author Matt Harris
		 */
		function admin_menu() {
			add_menu_page( 'Eventbrite', 'Eventbrite', 'manage_options', basename( __FILE__ ),
				[ $this, 'options_page' ], '' );
			$pagehook = add_submenu_page( basename( __FILE__ ), 'Eventbrite Plugin Settings', 'Settings',
				'manage_options', basename( __FILE__ ), [ $this, 'options_page' ] );
			add_submenu_page( basename( __FILE__ ), 'Eventbrite Plugin Help', 'Help', 'manage_options',
				basename( __FILE__ ) . '-help', [ $this, 'help_page' ] );

			// register a processing hook. We have to do this as the settings API doesn't handle site options
			add_action( "load-$pagehook", [ $this, 'update_options_page' ] );
			add_action( "admin_head-$pagehook", [ $this, 'admin_head_css' ] );
			add_action( "admin_head-$pagehook", [ $this, 'admin_head_js' ] );
			add_action( "admin_head-$pagehook", [ $this, 'admin_head_eb_js' ] );
		}

		function admin_head_css() { ?>
			<style type="text/css" media="screen">
                input.button-warning, button.button-warning, a.button-warning {
                    border-color: #b10000;
                    font-weight: bold;
                    color: #fff;
                    background: #b10000;
                    text-shadow: rgba(0, 0, 0, 0.3) 0 -1px 0;
                }

                input.button-warning:active, button.button-warning:active, a.button-warning:active {
                    background: #b10000;
                    color: #eaf2fa;
                }

                input.button-warning:hover, button.button-warning:hover, a.button-warning:hover {
                    background-color: red;
                    border-color: #990000;
                    color: #eaf2fa;
                }

                input#ticketfly_api_cache_store {
                    margin-right: 10px;
                }

                input.tfly-error {
                    outline: 3px solid red;
                }

                input.tfly-ok {
                    outline: 3px solid green;
                }

                span.tfly-error {
                    font-weight: bold;
                    color: red;
                }

                span.tfly-ok {
                    font-weight: bold;
                    color: green;
                }
			</style>
			<?php
		}

		function admin_head_js() { ?>
			<script type="text/javascript" charset="utf-8">
                jQuery(document).ready(function ($) {
                    var loading_div = function (id, text) {
                        var loadid = id + '_loading';
                        content = '<span id="' + loadid.replace('#', '') + '">' + text + '</span>';

                        if (jQuery(id).length) {
                            jQuery(id).after(content);
                            jQuery(loadid).hide();
                        }

                        return {
                            set_class: function (new_class) {
                                $(loadid).removeClass().addClass(new_class + ' tfly_loading');
                            },
                            set_text: function (text) {
                                $(loadid).text(text);
                            },
                            show: function (text) {
                                if (text) {
                                    this.set_text(text);
                                }
                                $(loadid).show();
                            },
                            hide: function () {
                                $(loadid).hide();
                            }
                        };
                    }
                    $id = '#ticketfly_api_cache_store';
                    $loading = new loading_div($id);

                    $('#purge_cache').live('click', function () {
                        $(this).attr('name', 'reallypurge');
                        return confirm('Are you sure you want to purge the cache?');
                    });
                    $($id).blur(function (event) {
                        $loading.show('checking...');
                        $.get('admin-ajax.php?action=ticketfly_store_ok&store=' + escape($(this).val()), function (data) {
                            if ('ok' == data) {
                                $loading.set_text('Cache store is ok.');
                                $loading.set_class('tfly-ok');
                                $($id).removeClass('tfly-error').addClass('tfly-ok');
                            } else {
                                $loading.set_text('Location is not usable as the cache store.');
                                $loading.set_class('tfly-error');
                                $($id).removeClass('tfly-ok').addClass('tfly-error');
                            }
                        })
                    }).focus(function (event) {
                        $loading.hide();
                        $($id).removeClass();
                    });
                });
			</script>
			<?php
		}

		function admin_head_eb_js() { ?>
			<script type="text/javascript" charset="utf-8">
                jQuery(document).ready(function ($) {
                    $('#ticketfly_plugin_enable_purchase_links').parent().addClass('purchase_tr');
                    var parentTr = $('.purchase_tr').parent();
                    if ($('#ticketfly_plugin_enable_purchase_modal').is(':checked')) {
                        parentTr.hide();
                    }
                    $('#ticketfly_plugin_enable_purchase_modal').click(function () {
                        if ($(this).is(':checked')) {
                            parentTr.hide(300);
                        } else {
                            parentTr.show(200);
                        }
                    });
                });
			</script>
			<?php
		}


		/**
		 * Retrieves the options from the site meta table and sets any missing
		 * values to their defaults
		 *
		 * @return array of the plugin options
		 * @author Marco Schmoecker
		 */
		function get_options() {
			$options = get_option( $this->settings_ref );
			$options = wp_parse_args( $options, $this->defaults );
			$options = sanitize_post( $options, 'db' );
			$options = array_intersect_key( $options, $this->defaults );

			return $options;
		}

		/**
		 * Validation and sanitisation for the options. PHP 4 compatible
		 *
		 * @param array $fields the key = value pairs to be validated and sanitised
		 *
		 * @return array the sanitised and validated fields.
		 * @author Matt Harris
		 */
		function options_validate( $fields ) {
			foreach ( $fields as $key => $value ) {
				if ( is_array( $value ) ) {
					$fields[ $key ] = array_filter( $value, 'sanitize_text_field' );
				} else {
					$fields[ $key ] = sanitize_text_field( $value ); // wp_filter_post_kses() expects slashed
				}
			}

			return $fields;
		}

		/**
		 * Renders the options page
		 *
		 * @return void
		 * @author Matt Harris
		 */
		function options_page() {
			if ( isset( $_GET[ 'error' ] ) ) {
				?>
				<div id="message" class="error fade">
					<p><?php echo esc_html( base64_decode( sanitize_text_field( $_GET[ 'error' ] ) ) ) ?></p>
				</div>
				<?php
			} elseif ( isset( $_GET[ 'updated' ] ) ) {
				?>
				<div id="message" class="updated fade"><p><?php echo esc_html__( 'Options saved.',
							'eventbrite' ) ?></p></div>
				<?php
			} elseif ( isset( $_GET[ 'deleted' ] ) ) {
				?>
				<div id="message" class="updated fade"><p><?php echo esc_html__( 'Plugin data deleted.',
							'eventbrite' ) ?></p></div>
				<?php
			} elseif ( isset( $_GET[ 'cancelled' ] ) ) {
				?>
				<div id="message" class="error fade"><p><?php echo esc_html__( 'Cancelled.', 'eventbrite' ) ?></p>
				</div>
				<?php
			} elseif ( isset( $_GET[ 'update-venues' ] ) ) {
				?>
				<div id="message" class="updated fade"><p><?php echo esc_html__( 'Venue IDs updated.',
							'eventbrite' ) ?></p></div>
			<?php } ?>

			<div class="wrap">
				<h2><?php esc_html__( 'Eventbrite Plugin', 'eventbrite' ); ?></h2>
				<form method="post"
					  action="<?php echo esc_url( admin_url( 'admin.php?page=' . basename( __FILE__ ) ) ); ?>">
					<?php
					settings_fields( $this->settings_ref );
					do_settings_sections( "{$this->settings_ref}_page" );
					?>
					<p class="submit">
						<input type="submit" name="submit" class="button-primary"
							   value="<?php echo esc_html__( 'Save Changes', 'eventbrite' ); ?>">
						<input id="cancel" type="submit" name="cancel" class="button-secondary"
							   value="<?php echo esc_html__( 'Undo Changes', 'eventbrite' ); ?>">
					</p>
				</form>
			</div>
			<?php
		}

		/**
		 * Renders the help page
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function help_page() {
			?>
			<div class="wrap">
				<?php include( "help.php" ); ?>
			</div>
			<?php
		}

		/**
		 * Verifies the user is an admin and the settings have been registered
		 * with WP before saving the options to the blog database as a
		 * serialized array.
		 *
		 * The code is the same as the settings API handler in wp-admin/options.php
		 * except we save to site_options instead of blog options.
		 *
		 * @return void
		 * @author Matt Harris & Marco Schmoecker
		 */
		function update_options_page() {

			// this is a POST back right?
			if ( isset( $_POST[ 'option_page' ] ) ) {
				// check the nonce
				$option_page = $_POST[ 'option_page' ];
				check_admin_referer( $option_page . '-options' );

				if ( isset( $_POST[ 'submit' ] ) ) {
					global $wp_version;
					if ( $wp_version > 5.5 ) {
						// run the whitelist filtering and verify our settings are in it
						$allowed_options = apply_filters( 'allowed_options', '' );
					} else {
						$allowed_options = apply_filters( 'whitelist_options', '' );
					}

					if ( ! isset( $allowed_options[ $option_page ] ) ) {
						wp_die( esc_html__( 'Error! Options page not found.', 'eventbrite' ) );
					}
					$options = $allowed_options[ $option_page ];

					$this->save_options( $options );
				} elseif ( isset( $_POST[ 'cancel' ] ) ) {
					$goback = add_query_arg( 'cancelled', 'true', wp_get_referer() );
					wp_redirect( $goback );
					exit();
				} elseif ( isset( $_POST[ 'update-venues' ] ) ) {
					$goback = add_query_arg( 'update-venues', 'true', wp_get_referer() );
					wp_redirect( $goback );
					exit();
				}
			}
		}

		/**
		 * Prepares the options for entry into the database and then saves them
		 * to the database. The function redirects the user back to the page they
		 * came from.
		 *
		 * @param array $options
		 */
		function save_options( $options = false ) {
			if ( $options ) {
				// clean and verify the options array.
				foreach ( $options as $option ) {
					$option = trim( $option );
					$value  = null;
					if ( isset( $_POST[ $option ] ) ) {
						$value = $_POST[ $option ];
					}
					if ( ! is_array( $value ) ) {
						$value = trim( $value );
					}
					$value = stripslashes_deep( $value );
					if ( isset( $value[ "use-venue-tickboxes" ] ) ) {
						if ( is_array( $value[ 'venue_id_tickboxes' ] ) ) {
							$value[ 'venue_id' ] = implode( ',', array_keys( $value[ 'venue_id_tickboxes' ] ) );
						} elseif ( ! isset( $value[ 'use-venue-tickboxe' ] ) ) {
							$value[ 'venue_id' ] = '';
						}
					}

					$settings = $this->get_options();

					$status = update_option( $option, $value );
					// any changes?
					if ( false !== $status ) { //&& ($option == 'radio_event_mode')
						//check if event mode changed -> flush rewrite rules
						if ( isset( $settings[ 'radio_event_mode' ] ) && isset( $value[ 'radio_event_mode' ] )
						     && ( $settings[ 'radio_event_mode' ] !== $value[ 'radio_event_mode' ] ) ) {
							update_option( "tfly_rewrite_done", 'needs update' );
						}
					}
				}

				// send us back with the updated flag set
				$goback = add_query_arg( 'updated', 'true', wp_get_referer() );
			} else {
				$goback = add_query_arg( 'error', 'true', wp_get_referer() );
			}
			wp_redirect( $goback );
			exit;
		}

		/**
		 * Change Page title for event, calendar & attraction pages
		 * //TODO: Is this working?
		 *
		 * @param string $title
		 *
		 * @return string $title
		 * @author Marco Schmoecker
		 */
		function eb_title( $title ) {
			global $post, $wp_query;

			if ( isset( $post->eb_events_page_role ) )
				switch ( $post->eb_events_page_role ) {
					case 'event_details':
						if ( isset( $post->eb_event ) ) {
							$title = eb_get_createEventPageTitle( $post->eb_event, $title );
						} else {
							$title = 'Event does not exist ';
						}
						$post->post_title = $title;
						break;

					case 'calendar':
						if ( isset( $post->eb_type ) && $post->eb_type === 'org' ) {
							$title = $this->get_orgTitle( $wp_query->query_vars[ 'eb_id' ] );
							$title .= ' Calendar';
						} elseif ( isset( $post->eb_type ) && $post->eb_type === 'venue' ) {
							$orgId = ( $post->eb_org_id ) ? $post->eb_org_id : '';
							$title = $this->get_venueTitle( $orgId, $wp_query->query_vars[ 'eb_id' ] );
							$title .= ' Calendar';
						}
						break;
					case 'attraction':
						// todo: adopt new EB API
						$title = $post->api_data->attraction->name . ' tickets, concerts and tour dates';
						break;
				}

			return ' ' . $title . ' ';
		}

		/**
		 * Enable link target for ticket purchase links?
		 *
		 * @param void
		 *
		 * @return boolean
		 */
		function enablePurchaseLinks() {
			$settings = $this->get_options();

			if ( isset( $settings[ 'enable_purchase_links' ] ) && ( false === $settings[ 'enable_purchase_links' ] ) ) {
				return false;
			}

			return true;
		}

		/**
		 * Inject share image meta tag into header of every page
		 *
		 * @param void
		 * todo: still working?
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function eb_fb_share_image() {
			global $post;

			if ( is_singular() &&
			     isset( $post->eb_events_page_role ) &&
			     ( $post->eb_events_page_role === 'event_details' ) &&
			     isset( $post->tfly_share_image )
			) {
				echo '<link rel="image_src" href="' . esc_url( $post->tfly_share_image ) . '" />';
			} elseif ( isset( $post->eb_events_page_role ) &&
			           ( 'attraction' === $post->eb_events_page_role ) &&
			           isset( $post->api_data->attractionInfo->image->original->path )
			) {
				echo sprintf( '<link rel="image_src" href="%s" />',
					esc_url_raw( $post->api_data->attractionInfo->image->original->path ) );
			}
		}

		/**
		 * Shortcode: Generating Site Name and ID for footer UTM code
		 */
		function eb_powered_func( $atts ) {
			$atts         = shortcode_atts( [
				'color' => 'orange',
			], $atts );
			$site_title   = get_bloginfo( 'name' );
			$eb_subdomain = sanitize_title_with_dashes( $site_title );

			$url = 'https://www.eventbrite.com/?utm_source=' . $eb_subdomain . '-' . get_current_blog_id() . '&utm_medium=poweredbyeventbritebadge';

			return '<article class="eb-powered-footer"><figure class="eb-powered">
                    <a href="' . esc_url( $url ) . '"><span class="powered-by ico--color-brand-' . esc_attr( $atts[ 'color' ] ) . '">Powered by </span>
                    <i class="ico-eventbrite-wordmark ico--color-brand-' . esc_attr( $atts[ 'color' ] ) . '"></i><span class="text">Eventbrite</span></a>
                    </figure></article>';

		}


		/**
		 * Shortcode: Calendar (lists events in calendar layout)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Yoli Hodde & Marco Schmoecker
		 */
		function eb_cal_func( $atts ) {
			global $post;
			extract( shortcode_atts( [
				'style'        => 'Style1',
				'org'          => '',
				'venue'        => '',
				'num'          => '50',
				'max'          => '400',
				'startofweek'  => 'Sunday',
				'daterange'    => '',
				'startdate'    => '',
				'enddate'      => '',
				'imagesize'    => 'medium1',
				'excludevenue' => '',
				'queryvars'    => '',
				'template'     => 'calendarClassic',
				'type'         => 'current_future',
			], $atts ) );

			$settings = $this->get_options();

			$req = [];

			if ( isset( $settings[ 'radio_filter_endpoint' ] ) ) {
				if ( 'venues' === $settings[ 'radio_filter_endpoint' ] ) {
					$req[ 'venues' ] = $settings[ 'venue_id' ];
				} else {
					$req[ 'organizers' ] = $settings[ 'org_id' ];
				}
			}

			if ( isset ( $venue ) && ! empty ( $venue ) ) {
				$req[ 'venues' ] = explode( ',', $venue );
				unset( $req[ 'organizers' ] ); //You can't have both
			}
			if ( isset ( $org ) && ! empty ( $org ) ) {
				$req[ 'organizers' ] = explode( ',', $org );
				unset( $req[ 'venues' ] ); //You can't have both
			}


			$req[ 'status' ]      = 'live';
			$req[ 'order_by' ]    = 'start_asc';
			$req[ 'page_size' ]   = (int) $num <= 50 ? (int) $num : 50;
			$req[ 'page' ]        = 1;
			$req[ 'time_filter' ] = 'current_future';
			$req[ 'expand' ]      = 'image,primary_organizer,venue,event_sales_status,ticket_availability,music_properties';

			$ebApiData = new Events_Api( $req );
			$ebApiData->request();
			$events = $ebApiData->get_events();

			if ( ! empty( $events ) ) {
				if ( ! isset( $post->eb_jsonld ) ) {
					$post->eb_jsonld = [];
				}
				foreach ( $events as $event ) {
					$post->eb_jsonld[] = eb_json_ld( $event );
				}
			}

			$templateFiles = 'calendar';
			ob_start();
			require eb_locate_template( $templateFiles );

			return ob_get_clean();
		}


		/**
		 * Shortcode: (Simple) Upcoming events listing (lists events in sidebar)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker & Yoli Hodde
		 */
		function eb_upcoming_func( $atts ) {
			global $post, $wp_query;
			extract( shortcode_atts( [
				'eventids' => '',
				'venue'    => '',
				'org'      => '',
				'num'      => '10',
				'title'    => '',
				'template' => '',
				'type'     => '',
			], $atts ) );

			$settings = $this->get_options();

			$events = [];
			if ( isset ( $eventids ) && ! empty ( $eventids ) ) {
				$event_api = new Event_Api();
				$eventids  = explode( ',', $eventids );
				foreach ( $eventids as $eventid ) {
					$events[] = $event_api->get_event( absint( $eventid ), [
						'expand' => 'eventbrite_event,primary_organizer,venue,ticket_availability',
					] );
				}
			}

			$req = [];

			if ( isset( $settings[ 'radio_filter_endpoint' ] ) ) {
				if ( 'venues' === $settings[ 'radio_filter_endpoint' ] ) {
					$req[ 'venues' ] = $settings[ 'venue_id' ];
				} else {
					$req[ 'organizers' ] = $settings[ 'org_id' ];
				}
			}

			if ( isset ( $venue ) && ! empty ( $venue ) ) {
				$req[ 'venues' ] = explode( ',', $venue );
				unset( $req[ 'organizers' ] ); //You can't have both
			}
			if ( isset ( $org ) && ! empty ( $org ) ) {
				$req[ 'organizers' ] = explode( ',', $org );
				unset( $req[ 'venues' ] ); //You can't have both
			}

			$req[ 'order_by' ]    = 'start_asc';
			$req[ 'page_size' ]   = ( (int) $num >= 20 ) ? 20 : (int) $num;
			$req[ 'page' ]        = 1;
			$req[ 'time_filter' ] = 'current_future';
			$req[ 'expand' ]      = 'eventbrite_event,primary_organizer,venue,ticket_availability';
			$req[ 'status' ]      = 'live';

			$ebApiData = new Events_Api( $req );
			if ( ! empty( $events ) ) {
				$ebApiData->set_events( $events );
			} else {
				$ebApiData->request();
			}


			ob_start();
			require eb_locate_template( 'upcoming' );

			return ob_get_clean();
		}


		/**
		 * Shortcode: Event details (lists event data on event detail page)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_details_func( $atts ) {
			global $post;

			extract( shortcode_atts( [
				'venue'   => '',
				'org'     => '',
				'eventid' => '',
			], $atts ) );

			$settings = $this->get_options();

			ob_start();
			require eb_locate_template( 'event_details' );

			return ob_get_clean();
		}


		/**
		 * Shortcode: Event search form
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_searchForm_func( $atts ) {
			global $wp_query;

			extract( shortcode_atts( [
				'search_results_page'        => '',
				'formtext'                   => 'Enter an Artist or Event',
				'hide_searchbox_richsnippet' => 'no',
			], $atts ) );

			if ( $search_results_page === '' ) {
				$search_results_page = eb_URL( 'listing' );
			}
			$wp_query->tfly_search_page = $search_results_page;

			if ( 'no' === $hide_searchbox_richsnippet ) {
				add_action( 'wp_footer', 'eb_search_jsonld_markup' );
			}

			ob_start();
			require eb_locate_template( 'searchForm' );

			return ob_get_clean();
		}

		/**
		 * Shortcode: Email subscribe form
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_emailForm_func( $atts ) {
			return ''; //Deprecated
		}

		/**
		 * Shortcode: Carousel (show up to 10 events in a carousel)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_carousel_func( $atts ) {
			global $post, $wp_query;
			extract( shortcode_atts( [
				'eventids'     => '',
				'venue'        => '',
				'org'          => '',
				'num'          => '4',
				'type'         => '',
				'dateformat'   => '',
				'excludevenue' => '',
				'exclude'      => '',
				'template'     => '',
				'queryvars'    => '',
			], $atts ) );

			wp_enqueue_script( 'owl-carousel-js' );
			wp_enqueue_script( 'owl-carousel-aria' );
			wp_enqueue_style( 'owl-carousel' );
			wp_enqueue_style( 'owl-theme' );

			if ( '' !== $queryvars ) {
				$pairs = explode( ',', $queryvars );
				foreach ( $pairs as $pair ) {
					$mapping = explode( '=', $pair );
					if ( isset( $mapping[ 0 ] ) && isset( $mapping[ 1 ] ) && isset( $wp_query->query_vars[ $mapping[ 0 ] ] ) && ( null !== $wp_query->query_vars[ $mapping[ 0 ] ] ) ) {
						${$mapping[ 1 ]} = $wp_query->query_vars[ $mapping[ 0 ] ];
					}
				}
			}
			$expand = 'logo,venue,event_sales_status,ticket_availability,music_properties';
			$events = [];
			if ( isset ( $eventids ) && ! empty ( $eventids ) ) {
				$event_api = new Event_Api();
				$eventids  = explode( ',', $eventids );
				$eventids  = array_splice( $eventids, 0, 10 ); //Limit Eventids to 10 for carousel
				foreach ( $eventids as $eventid ) {
					$events[] = $event_api->get_event( absint( $eventid ), [
						'expand' => $expand,
					] );
				}
			}

			$settings           = $this->get_options();
			$req                = [];
			$req[ 'order_by' ]  = 'start_asc';
			$req[ 'page' ]      = 1;
			$req[ 'page_size' ] = (int) $num >= 10 ? 10 : (int) $num;

			if ( isset( $settings[ 'radio_filter_endpoint' ] ) ) {
				if ( 'venues' === $settings[ 'radio_filter_endpoint' ] ) {
					$req[ 'venues' ] = $settings[ 'venue_id' ];
				} else {
					$req[ 'organizers' ] = $settings[ 'org_id' ];
				}
			}

			if ( isset ( $venue ) && ! empty ( $venue ) ) {
				$req[ 'venues' ] = explode( ',', $venue );
				unset( $req[ 'organizers' ] ); //You can't have both
			}
			if ( isset ( $org ) && ! empty ( $org ) ) {
				$req[ 'organizers' ] = explode( ',', $org );
				unset( $req[ 'venues' ] ); //You can't have both
			}

			$req[ 'time_filter' ] = 'current_future';
			if ( $type && $type === 'justannounced' ) {
				$req[ 'order_by' ]    = 'created_desc';
				$req[ 'time_filter' ] = 'current_future';
			}
			$req[ 'status' ] = 'live';
			$req[ 'expand' ] = $expand;
			$req[ 'page' ]   = 1;

			$ebApiData = new Events_Api( $req );
			if ( ! empty( $events ) ) {
				$ebApiData->set_events( $events );
			} else {
				$ebApiData->request();
			}
			$events = $ebApiData->get_events( true );

			if ( ! empty( $events ) ) {
				if ( ! isset( $post->eb_jsonld ) ) {
					$post->eb_jsonld = [];
				}
				foreach ( (array) $events as $event ) {
					$post->eb_jsonld[] = eb_json_ld( $event );
				}
			}

			$templateFiles = [ $template, 'carousel' ];

			ob_start();
			require eb_locate_template( $templateFiles );

			return ob_get_clean();
		}

		/**
		 * Shortcode: Simple event listing of just announced events (used in sidebar)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_justAnnounced_func( $atts ) {
			global $post;
			extract( shortcode_atts( [
				'eventids' => '',
				'venue'    => '',
				'org'      => '',
				'num'      => '10',
				'title'    => '',
				'type'     => '',
			], $atts ) );

			$settings = $this->get_options();
			$req      = [];
			$expand   = 'primary_organizer,venue,ticket_availability,event_sales_status';
			$events   = [];
			if ( isset ( $eventids ) && ! empty ( $eventids ) ) {
				$event_api = new Event_Api();
				$eventids  = explode( ',', $eventids );
				$eventids  = array_splice( $eventids, 0, 20 ); //Limit Eventids to 10 for carousel
				foreach ( $eventids as $eventid ) {
					$events[] = $event_api->get_event( absint( $eventid ), [
						'expand' => $expand,
					] );
				}
			}

			if ( isset( $settings[ 'radio_filter_endpoint' ] ) ) {
				if ( 'venues' === $settings[ 'radio_filter_endpoint' ] ) {
					$req[ 'venues' ] = $settings[ 'venue_id' ];
				} else {
					$req[ 'organizers' ] = $settings[ 'org_id' ];
				}
			}

			if ( isset ( $venue ) && ! empty ( $venue ) ) {
				$req[ 'venues' ] = explode( ',', $venue );
				unset( $req[ 'organizers' ] ); //You can't have both
			}
			if ( isset ( $org ) && ! empty ( $org ) ) {
				$req[ 'organizers' ] = explode( ',', $org );
				unset( $req[ 'venues' ] ); //You can't have both
			}

			$req[ 'page' ]        = 1;
			$req[ 'order_by' ]    = 'created_desc';
			$req[ 'page_size' ]   = ( (int) $num >= 20 ) ? 20 : (int) $num;
			$req[ 'time_filter' ] = 'current_future';
			$req[ 'expand' ]      = $expand;
			$req[ 'status' ]      = 'live';

			$ebApiData = new Events_Api( $req );
			if ( ! empty( $events ) ) {
				$ebApiData->set_events( $events );
			} else {
				$ebApiData->request();
			}
			ob_start();
			require eb_locate_template( 'justAnnounced' );

			return ob_get_clean();

		}


		/**
		 * Shortcode: Simple upcoming on sales listing (used in sidebar)
		 *
		 * @param array (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function tf_onSale_func( $atts ) {
			echo ''; //deprecated

		}


		/**
		 * Shortcode: Hero List view (lists events in list view layout)
		 *
		 * @param array $atts (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_heroListView_func( $atts ) {
			global $post, $wp_query;
			extract(
				shortcode_atts(
					[
						'eventids'      => '',
						'venue'         => '',
						'org'           => '',
						'title'         => 'Upcoming Shows',
						'num'           => '50',
						'type'          => '',
						'consecutive'   => '',
						'excludevenue'  => '',
						'daterange'     => '',
						'startdate'     => '',
						'enddate'       => '',
						'template'      => '',
						'queryvars'     => '',
						'country'       => '',
						'stateProvince' => '',
						'metro'         => '',
						'pagination'    => 1,
						'offset'        => '0',
					], $atts
				)
			);

			//It's possible if the shortcode isn't passed any attributes for $atts to be an empty string
			//We want to reset it and make sure it's an array if that is the case.
			if ( ! is_array( $atts ) ) {
				$atts = [];
			}
			//force shortcode (markup) template
			$atts[ 'template' ] = ( $template ) ? $template : 'heroListView';

			return $this->eb_listView_func( $atts );
		}


		function advConsItem( $events, $consecutive ) {
			if ( $consecutive === '' || 0 ) {
				$consecutive = 0;
			}
			$advDateLimiter = $consecutive;

			$currentArrayCounter = $this->findFirstUndone( $events );
			if ( $currentArrayCounter === false ) {
				return ( $events );
			}

			$eventNum                             = count( $events );
			$activeHeadlinerId                    = $events[ $currentArrayCounter ]->headliners[ 0 ]->id;
			$events[ $currentArrayCounter ]->done = true;

			for ( $i = $currentArrayCounter + 1; $i < $eventNum; $i ++ ) {
				$headlinerId = (int) $events[ $i ]->headliners[ 0 ]->id;

				if ( ( $headlinerId === $activeHeadlinerId ) && ( $venueId === $activeVenueId ) ) {

					$countTheDays = $this->advDateLimit( $events[ $currentArrayCounter ]->startDate,
						$events[ $i ]->startDate );
					if ( $countTheDays >= $advDateLimiter ) {
						break;
					}

					$consecutiveEvent                     = [];
					$consecutiveEvent[ 'id' ]             = $events[ $i ]->id;
					$consecutiveEvent[ 'startDate' ]      = $events[ $i ]->startDate;
					$consecutiveEvent[ 'doorsDate' ]      = $events[ $i ]->doorsDate;
					$consecutiveEvent[ 'topLineInfo' ]    = $events[ $i ]->topLineInfo;
					$consecutiveEvent[ 'supportsName' ]   = $events[ $i ]->supportsName;
					$consecutiveEvent[ 'ageLimit' ]       = $events[ $i ]->ageLimit;
					$consecutiveEvent[ 'additionalInfo' ] = $events[ $i ]->additionalInfo;

					// add consecutive event array to master event
					if ( ! isset( $events[ $currentArrayCounter ]->consecutiveEvents ) ) {
						$events[ $currentArrayCounter ]->consecutiveEvents = [];
					}
					array_push( $events[ $currentArrayCounter ]->consecutiveEvents, $consecutiveEvent );

					$events[ $i ]->hide = true;
				}
			}

			return $events;
		}


		function attractionItem( $events, $consecutive ) {
			if ( $consecutive === '' || 0 ) {
				$consecutive = 0;
			}
			$advDateLimiter      = $consecutive;
			$currentArrayCounter = $this->findFirstUndone( $events );
			if ( $currentArrayCounter === false ) {
				return ( $events );
			}
			$eventNum                             = count( $events );
			$activeHeadlinerId                    = $events[ $currentArrayCounter ]->headliners[ 0 ]->id;
			$events[ $currentArrayCounter ]->done = true;
			for ( $i = $currentArrayCounter + 1; $i < $eventNum; $i ++ ) {
				$headlinerId = (int) $events[ $i ]->headliners[ 0 ]->id;
				if ( $headlinerId === $activeHeadlinerId ) {
					$countTheDays = $this->advDateLimit( $events[ $currentArrayCounter ]->startDate,
						$events[ $i ]->startDate );
					if ( $countTheDays >= $advDateLimiter ) {
						break;
					}
					$attractionListItem                  = [];
					$attractionListItem[ 'id' ]          = $events[ $i ]->id;
					$attractionListItem[ 'startDate' ]   = $events[ $i ]->startDate;
					$attractionListItem[ 'topLineInfo' ] = $events[ $i ]->topLineInfo;
					// add consecutive event array to master event
					if ( ! isset( $events[ $currentArrayCounter ]->attractionListItems ) ) {
						$events[ $currentArrayCounter ]->attractionListItems = [];
					}
					array_push( $events[ $currentArrayCounter ]->attractionListItems, $attractionListItem );
					$events[ $i ]->hide = true;
				}
			}

			return $events;
		}

		function findFirstUndone( $events ) {
			$eventNum = count( $events );
			for ( $i = 0; $i < $eventNum; $i ++ ) {
				if ( ! isset( $events[ $i ]->done ) && ! isset( $events[ $i ]->hide ) ) {
					return ( $i );
				}
			}

			return false;
		}

//		   show events by same artist within a certain number of days
		function advDateLimit( $currentArrayCounter, $events ) {
			$thisNewDateValue = new DateTime();
			$thisNewDateValue->setTimestamp( strtotime( date( 'Y-m-d 0:0:0',
				strtotime( $currentArrayCounter ) ) ) );
			$nextNewDateValue = new DateTime();
			$nextNewDateValue->setTimestamp( strtotime( date( 'Y-m-d 0:01:01', strtotime( $events ) ) ) );

			return $thisNewDateValue->diff( $nextNewDateValue )->format( '%a' );
		}

		/**
		 * Shortcode: EB List view (lists events in list view layout)
		 *
		 * @param array $atts (see $atts below)
		 *
		 * @return string (buffer contents: rendered markup)
		 * @author Marco Schmoecker
		 */
		function eb_listView_func( $atts ) {
			global $post, $wp_query;
			extract( shortcode_atts( [
				'eventids'   => '',
				'venue'      => '',
				'org'        => '',
				'collection' => '',
				'title'      => 'Upcoming Shows',
				'num'        => '50',
				'type'       => 'current_future',
				'daterange'  => '',
				'startdate'  => '',
				'enddate'    => '',
				'template'   => '',
				'queryvars'  => '',
				'pagination' => 1,
				'offset'     => '0',
			], $atts ) );

			if ( '' !== $queryvars ) {
				$pairs = explode( ',', $queryvars );
				foreach ( $pairs as $pair ) {
					$mapping = explode( '=', $pair );
					if ( isset( $mapping[ 0 ] ) && isset( $mapping[ 1 ] ) && isset( $wp_query->query_vars[ $mapping[ 0 ] ] ) && ( null !== $wp_query->query_vars[ $mapping[ 0 ] ] ) ) {
						${$mapping[ 1 ]} = $wp_query->query_vars[ $mapping[ 0 ] ];
					}
				}
			}

			$settings = $this->get_options();
			$req      = [];

			if ( isset( $wp_query->query_vars[ 'page' ] )
			     && is_int( (int) $wp_query->query_vars[ 'page' ] )
			) {
				$page = $wp_query->query_vars[ 'page' ];
			} elseif ( isset( $wp_query->query_vars[ 'paged' ] )
			           && is_int( (int) $wp_query->query_vars[ 'paged' ] )
			) {
				$page = $wp_query->query_vars[ 'paged' ];
			} elseif ( isset( $_GET[ 'eb_page' ] ) ) {
				$page = sanitize_text_field( $_GET[ 'eb_page' ] );
			} else {
				$page = 1;
			}
			$singlePage        = 'false';
			$req[ 'page' ]     = $page ? $page : 1;
			$req[ 'order_by' ] = 'start_asc';
			$req[ 'status' ]   = 'live';

			if ( $type && $type === 'justannounced' ) {
				$req[ 'order_by' ]    = 'created_desc';
				$req[ 'time_filter' ] = 'current_future';
			}

			$expand = 'image,primary_organizer,venue,event_sales_status,ticket_availability,music_properties';
			$events = [];
			if ( isset ( $eventids ) && ! empty ( $eventids ) ) {
				$event_api = new Event_Api();
				$eventids  = explode( ',', $eventids );
				$eventids  = array_splice( $eventids, 0, 50 ); //Limit Eventids to 10 for carousel
				foreach ( $eventids as $eventid ) {
					$events[] = $event_api->get_event( absint( $eventid ), [
						'expand' => $expand,
					] );
				}
			}

			if ( isset( $settings[ 'radio_filter_endpoint' ] ) ) {
				if ( 'venues' === $settings[ 'radio_filter_endpoint' ] ) {
					$req[ 'venues' ] = $settings[ 'venue_id' ];
				} else {
					$req[ 'organizers' ] = $settings[ 'org_id' ];
				}
			}

			if ( isset ( $venue ) && ! empty ( $venue ) ) {
				$req[ 'venues' ] = explode( ',', $venue );
				unset( $req[ 'organizers' ] ); //You can't have both
			}
			if ( isset ( $org ) && ! empty ( $org ) ) {
				$req[ 'organizers' ] = explode( ',', $org );
				unset( $req[ 'venues' ] ); //You can't have both
			}

			$req[ 'page' ] = $page ? $page : 1;

			$req[ 'page_size' ]   = ( (int) $num >= 50 || $num === '0' ) ? 50 : (int) $num;
			$req[ 'time_filter' ] = 'current_future';
			$req[ 'expand' ]      = $expand;

			if ( isset( $_GET[ 'q' ] ) ) {
				$req[ 'order_by' ]    = 'start_asc';
				$req[ 'name_filter' ] = sanitize_text_field( $_GET[ 'q' ] );
			}

			$ebApiData = new Events_Api( $req );

			if ( ! empty( $events ) ) {
				$ebApiData->set_events( $events );
			} else {
				$ebApiData->request();
			}

			$events = $ebApiData->get_events( true );

			if ( ! empty( $events ) ) {
				if ( ! isset( $post->eb_jsonld ) ) {
					$post->eb_jsonld = [];
				}
				foreach ( $events as $event ) {
					$post->eb_jsonld[] = eb_json_ld( $event );
				}
			}

			$templateFiles = [ $template, 'listView' ];

			ob_start();
			require eb_locate_template( $templateFiles );

			return ob_get_clean();
		}

		/**
		 * Determine the API endpoint
		 *
		 * @param array $settings app settings
		 *
		 * @return string path API end point path/function
		 * @author Marco Schmoecker
		 */
		function get_api_endpoint( $settings ) {
			if ( ( isset( $_GET[ 'q' ] ) ) || ( isset( $settings[ 'radio_endpoint_mode' ] ) && ( 'organizers' === $settings[ 'radio_endpoint_mode' ] ) ) ) {
				return 'search';
			} elseif ( isset( $settings[ 'radio_endpoint_mode' ] ) && ( 'collection' === $settings[ 'radio_endpoint_mode' ] ) ) {
				return 'collections';
			}

			return 'organizers';
		}


		/**
		 * Inject GTM container
		 *
		 * @param void
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */

		function eb_printGTM() {
			echo "
<!-- Google Tag Manager -->
<script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start':
new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0],
j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
'//www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
})(window,document,'script','dataLayer','GTM-TWWLNB');</script>
<!-- End Google Tag Manager -->
";
		}

		/**
		 * Inject GA tracking JS
		 *
		 * @param void
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function eb_printGaTracking() {
			$settings = $this->get_options();
			if ( ( isset( $settings[ "ga_enable_ganalytics" ] ) && $settings[ "ga_enable_ganalytics" ] ) ||
			     ( isset( $settings[ 'enable_purchase_modal' ] ) ) && ( true === $settings[ 'enable_purchase_modal' ] ) ) {
				$ga_ua_code = ( isset( $settings[ 'ga_ua_code' ] ) && ( ! empty( $settings[ 'ga_ua_code' ] ) ) )
					? $settings[ 'ga_ua_code' ] : '';
				$ga_gtm_js  = 'https://www.googletagmanager.com/gtag/js?id=' . $ga_ua_code;
				wp_enqueue_script( 'googleGTM-async', $ga_gtm_js, '', EB_VERSION, false );
				echo '
<script>(function(i,s,o,g,r,a,m){i["GoogleAnalyticsObject"]=r;i[r]=i[r]||function(){
(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
})(window,document,"script","https://www.google-analytics.com/analytics.js","ga");
ga("create", ' . wp_json_encode( $ga_ua_code ) . ', "auto");
ga("send", "pageview");</script>
<script>window.dataLayer = window.dataLayer || [];
function gtag(){dataLayer.push(arguments);}
gtag("js", new Date());
gtag("config", ' . wp_json_encode( $ga_ua_code ) . ');</script>';
			}
		}

		/**
		 * Hooks into WP "init" filter
		 * Future dev: Do we really need to do a flush_rules, or, are we able to add in our rw rules on the fly?
		 *
		 * @param void
		 *
		 * @return void
		 * @author Matt Harris, Marco Schmoecker
		 */
		function init() {

			if ( get_option( "tfly_rewrite_done", null ) !== self::$pluginVersion ) {
				flush_rewrite_rules();
				update_option( "tfly_rewrite_done", self::$pluginVersion );
			}

			if ( ! is_admin() ) {
				wp_enqueue_script( 'eb', eb_plugin_dir_URL() . '/js/eb.js', [ 'jquery-ui-dialog' ], EB_VERSION,
					true );
			}

			add_action( 'admin_head', 'replace_eb_admin_menu_icon' );
			function replace_eb_admin_menu_icon() {
				?>
				<style type="text/css">
                    #adminmenu #toplevel_page_eb-events .dashicons-admin-generic:before {
                        content: '';
                        background: url('/wp-content/plugins/eventbrite-events/images/ticket.svg') 9px no-repeat;
                        background-size: 20px 20px;
                        display: block;
                        width: 30px;
                    }
				</style>
			<?php }

		}

		/**
		 * Enable/whitelist query vars
		 *
		 * @param array $vars
		 *
		 * @return array
		 * @author Matt Harris, Marco Schmoecker
		 */
		function query_vars( $vars ) {
			// We need to whitelist our query vars to ensure they're accessible to other functions
			array_push( $vars, 'eb_id' );        // venue id(s)
			array_push( $vars, 'eb_org_id' );    // org id(s)
			array_push( $vars, 'eb_type' );    // org id(s)
			array_push( $vars, 'num_results' );    // max # of results to request from API per page

			return $vars;
		}

		/**
		 * Add in our rewrite rules
		 *
		 * @param array $rules
		 *
		 * @return array $new_rules
		 * @author Matt Harris, Marco Schmoecker
		 */
		function rewrite_rules_array( $rules ) {
			$newrules = [];

			// TODO - these slugs should come from the options table as set when the 'page roles' are defined
			$slugs                                                                                               = '(venue|org|calendar|listing)';
			$newrules[ '(calendar)/([0-9]{2})/([0-9]{4})/?$' ]                                                   = 'index.php?pagename=$matches[1]&calmonth=$matches[2]&calyear=$matches[3]';
			$newrules[ '(calendar)/(venue|org)/([0-9]{1,})/?$' ]                                                 = 'index.php?pagename=$matches[1]&eb_type=$matches[2]&eb_id=$matches[3]';
			$newrules[ $slugs . '/([0-9]{1,})?(-.*)?/?$' ]                                                       = 'index.php?pagename=$matches[1]&eb_id=$matches[2]';
			$newrules[ $slugs . '/([0-9]{1,})?(-.*)?/page/([0-9]{1,})/?$' ]                                      = 'index.php?pagename=$matches[1]&eb_id=$matches[2]&page=$matches[3]';
			$newrules[ 'feed/ical/?$' ]                                                                          = 'index.php?eb_type=ical'; ///wp-content/plugins/eventbrite-events/includes/ical.php';
			$newrules[ '(event|e|spotlight-event|venue|org|calendar|listing)/(.*-)?([0-9]{1,})?/?feed/ical/?$' ] = 'index.php?pagename=$matches[1]&eb_id=$matches[3]&eb_type=ical';
			$newrules[ '(event|e)/(.*-)?([0-9]{1,})/stub/?$' ]                                                   = 'index.php?pagename=$matches[1]&eb_id=$matches[3]&eb_type=stub';
			$newrules[ '(event|e|spotlight-event)/(.*-)?([0-9]+)/?$' ]                                           = 'index.php?pagename=$matches[1]&eb_id=$matches[3]';

			$rules = $newrules + $rules;

			$smRules = [
				'sitemap(-+([a-zA-Z0-9_-]+))?\.xml$'     => 'index.php?xml_sitemap=params=$matches[2]',
				'sitemap(-+([a-zA-Z0-9_-]+))?\.xml\.gz$' => 'index.php?xml_sitemap=params=$matches[2];zip=true',
				'sitemap(-+([a-zA-Z0-9_-]+))?\.html$'    => 'index.php?xml_sitemap=params=$matches[2];html=true',
				'sitemap(-+([a-zA-Z0-9_-]+))?\.html.gz$' => 'index.php?xml_sitemap=params=$matches[2];html=true;zip=true',
			];

			return array_merge( $smRules, $rules );
		}

		/**
		 * Hook into request action and include a custom template for ical feed and event detail stub page
		 *
		 * @param object $wp
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function eb_parse_request( &$wp ) {
			if ( array_key_exists( 'eb_type', $wp->query_vars ) ) {
				if ( ( $wp->query_vars[ 'eb_type' ] === 'ical' ) ) {
					include( dirname( __FILE__ ) . '/includes/ical.php' );
					exit();
				} elseif ( ( $wp->query_vars[ 'eb_type' ] === 'stub' ) ) {
					include( dirname( __FILE__ ) . '/includes/eventDetailStub.php' );
					exit();
				}
			}
		}

		/**
		 * Add in our JS on WP admin pages to enabled short code injection into the editor
		 *
		 * @param void
		 *
		 * @return void
		 * @author Marco Schmoecker
		 */
		function admin_head() {
			add_meta_box(
				$this->id,
				__( 'Page Role' ),
				[ $this, 'meta_box_page_role' ],
				'page',
				'side',
				'high'
			);
			?>
			<script type="text/javascript">

                function returnObjById(id) {
                    if (document.getElementById)
                        var returnVar = document.getElementById(id);
                    else if (document.all)
                        var returnVar = document.all[id];
                    else if (document.layers)
                        var returnVar = document.layers[id];
                    return returnVar;
                }

                function injectShortcode(value) {

                    url = '';

                    switch (value) {
                        case 'calendar':
                            url = 'Calendar';
                            break;
                        case 'listing':
                            url = 'Listing';
                            value = 'listview';
                            break;
                        case 'venue_details':
                            url = 'Venue';
                            value = 'listview';
                            break;
                        case 'org_details':
                            url = 'Org';
                            value = 'listview';
                            break;
                        case 'event_details':
                            url = 'Event';
                            break;
                        case 'attraction':
                            url = 'Artist';
                            value = 'artistpage';
                            break;
                        default:
                            value = '';
                            url = '';
                    }

                    if (url && tinyMCE.activeEditor) {
                        returnObjById('title').value = url;
                        tinyMCE.activeEditor.setContent('');
                        send_to_editor('[eb_' + value + ']');
                        returnObjById('shortcodetext').innerHTML = '[eb_' + value + ']';
                        returnObjById('pageroleurl').innerHTML = '/' + url.toLowerCase() + '/';
                        return;
                    }
                    returnObjById('shortcodetext').innerHTML = '';
                    returnObjById('pageroleurl').innerHTML = '';
                    return;
                }
			</script>
			<?php
		}

		/**
		 * Custom function to search for & return a part of a string
		 *
		 * @param string $haystack
		 * @param string $needle
		 * @param bool   $before_needle
		 * @param bool   $include_needle
		 * @param bool   $case_sensitive
		 *
		 * @return string $substring or bool, if not found
		 * @author Marco Schmoecker
		 */
		function strstrbi(
			$haystack,
			$needle,
			$before_needle = false,
			$include_needle = true,
			$case_sensitive = false
		) {
			//Find the position of $needle
			if ( $case_sensitive ) {
				$pos = strpos( $haystack, $needle );
			} else {
				$pos = strpos( strtolower( $haystack ), strtolower( $needle ) );
			}

			//If $needle not found, abort
			if ( $pos === false ) {
				return false;
			}

			//Adjust $pos to include/exclude the needle
			if ( $before_needle === $include_needle ) {
				$pos += strlen( $needle );
			}

			//get everything from 0 to $pos?
			if ( $before_needle ) {
				return substr( $haystack, 0, $pos );
			}

			//otherwise, go from $pos to end
			return substr( $haystack, $pos );
		}

		function get_shortcode( $content, $tag ) {
			if ( false === strpos( $content, '[' ) ) {
				return false;
			}

			if ( shortcode_exists( $tag ) ) {
				preg_match_all( '/' . get_shortcode_regex() . '/', $content, $matches, PREG_SET_ORDER );
				if ( empty( $matches ) ) {
					return false;
				}

				foreach ( $matches as $shortcode ) {
					if ( $tag === $shortcode[ 2 ] ) {
						return $shortcode;
					} elseif ( ! empty( $shortcode[ 5 ] ) && has_shortcode( $shortcode[ 5 ], $tag ) ) {
						return $shortcode;
					}
				}
			}

			return false;
		}

		/**
		 * Hook into the_posts to prepare and add API & other data to the $post object
		 *
		 * @param array $posts
		 *
		 * @return array $posts
		 * @author Matt Harris, Marco Schmoecker
		 */
		function the_posts( $posts ) {
			global $wp_query;
			// the is_singular checks we only have one post but to ensure we don't
			// break if the indicies are not numerically indexed we still do a
			// foreach loop
			if ( is_singular() ) :

				$settings = $this->get_options();
				$eb_type  = get_query_var( 'eb_type' );

				foreach ( $posts as &$post ) :

					if ( strtolower( $post->post_title ) === 'venue' && ! isset( $eb_type ) ) {
						$post->eb_type = 'venue';
					}

					if ( ! isset( $post->eb_events_page_role ) || empty( $post->eb_events_page_role ) ) {
						if ( has_shortcode( $post->post_content, 'tf_attraction_details' ) ) {
							$post->eb_events_page_role = 'attraction';
						} elseif ( has_shortcode( $post->post_content, 'eb_event_details' ) ) {
							$post->eb_events_page_role = 'event_details';
						}
					}

					if ( ! isset( $post->eb_events_page_role ) ) {
						continue;
					}

					eb_widgetJs( $settings );

					$post->eb_id     = get_query_var( 'eb_id' );
					$post->eb_org_id = $settings[ 'org_id' ];
					if ( $eb_type ) {
						$post->eb_type = $eb_type;
					}
					$max = ! empty( $wp_query->query_vars[ 'num_results' ] ) ? $wp_query->query_vars[ 'num_results' ] : 200;

					switch ( $post->eb_events_page_role ) {
						case 'event_details':
							if ( ( false !== strpos( $post->post_content, "venue=" ) )
							     || ( false !== strpos( $post->post_content, "event=" ) ) ) {
								$shortcode = $this->get_shortcode( $post->post_content, 'eb_event_details' );
								if ( ( false !== $shortcode ) && isset( $shortcode[ 3 ] ) ) {
									$atts = shortcode_atts( [
										'event' => '',
										'venue' => ( $settings[ 'venue_id' ] ) ? $settings[ 'venue_id' ] : null,
									], shortcode_parse_atts( $shortcode[ 3 ] ), 'eb_event_details' );
								}
							} else {
								$atts = [];
							}
							$event_id = isset( $post->eb_id ) ? $post->eb_id : $atts[ 'event' ];

							$event_api       = new Event_Api();
							$performers_api  = new Performer_Api();
							$description_api = new Description_Api();

							$post->eb_event = $event_api->get_event( $event_id, [
								'expand' => 'event,image,primary_organizer,venue,event_sales_status,ticket_availability,music_properties',
							] );

							$post->description = $description_api->get_event_description( $event_id );

							$post->performances               = $performers_api->get_top_performers( $event_id );
							$post->eb_event->top_performances = $post->performances;

							// create json-ld data
							if ( isset( $post->eb_event ) && ! is_wp_error( $post->eb_event ) ) {
								if ( ! isset( $post->eb_jsonld ) ) {
									$post->eb_jsonld = [];
								}
								$post->eb_jsonld[] = eb_json_ld( $post->eb_event );
							}
							break;

						case 'calendar':
							break;
						case 'listing':
							break;
					}
				endforeach;
			endif;

			return $posts;
		}


		/**
		 * Process the URL of a page role page on save
		 *
		 * @param string $post_ID the ID of the post to be updated or that was just saved
		 * @param string $post the post object
		 */
		function update_pagerole_url( $post_ID ) {
			global $wpdb;
			if ( ! wp_is_post_revision( $post_ID ) ) {
				$post     = get_post( $post_ID );
				$post_url = $post->guid;
				if ( $post_url && isset( $_POST[ 'eb_events_page_role' ] ) ) {
					$variable = [ $_POST[ 'eb_events_page_role' ] => $post_url, 'opt02' => '2' ];
					update_option( 'ticketfly_plugin_urls', $variable );
				}

			}
		}

		/**
		 * Process the post meta on save
		 *
		 * @param string $post_ID the ID of the post to be updated or that was just saved
		 * @param string $post the post object
		 */
		function save_post( $post_ID, $post ) {
			// parse the args through the WordPress parsing function and sanitize
			$postarr = wp_parse_args( $_POST, $this->fields );
			$postarr = sanitize_post( $postarr, 'db' );
			// drop fields we don't want
			$postarr = array_intersect_key( $postarr, $this->fields );
			// only save the fields which are different. This also catches any instances
			// where the fields are not added to a page or custom post type (WP 3)
			$diffs = array_diff( $postarr, $this->fields );
			// save to the database, renaming all keys from 'key' to '_key'
			foreach ( $diffs as $k => $v ) {
				if ( empty( $v ) ) {
					delete_post_meta( $post_ID, "_$k" );
				} else {
					add_post_meta( $post_ID, "_$k", $v, true ) or update_post_meta( $post_ID, "_$k", $v );
				}
			}
		}

		/**
		 * Filter all post results and add the custom fields
		 */
		function posts_results( $posts ) {
			if ( is_array( $posts ) ) // for every post returned from the database query
			{
				foreach ( $posts as $k => $post ) {
					if ( $post->post_type !== 'post' ) // update the post so that it contains our meta data
					{
						$posts[ $k ] = $this->load_meta( $post );
					}
				}
			}

			// remember to return the $posts
			return $posts;
		}

		/**
		 * Add our extra fields to the post being edited
		 */
		function load_meta( $post ) {
			// extract the post ID
			$id = $post->ID;
			// for each of our fields, grab the meta data from the cached array and
			// create a new variable on the post object
			foreach ( $this->fields as $k => $v ) {
				// this line will turn a meta field like _tmh_portfolio_client
				// into $post->tmh_portfolio_client
				$post->{"$k"} = get_post_meta( $id, "_$k", true );
			}

			return $post;
		}

		/**
		 * Add our meta box widget to the post edit screen
		 */
		function meta_box_page_role( $post ) {
			// load each of our fields so we can display them in the meta box


			foreach ( $this->fields as $field_name => $field_value ) {
				$$field_name = get_post_meta( $post->ID, "_$field_name", true );

				if ( 'eb_events_page_role' === $field_name ) : ?>
					<fieldset>
						<ul role="radiogroup">
							<?php
							foreach ( $this->roles as $role_name => $role_value ) :
								$checked = checked( $$field_name, $role_name, false );
								$aria = $$field_name === $role_name ? 'true' : 'false'; ?>
								<li>
									<?php echo sprintf( '<input type="radio" role="radio" aria-checked="%s" name="%s" id="%s" value="%s" %s onclick="injectShortcode(this.value)" />',
										esc_attr( $aria ), esc_attr( $field_name ),
										esc_attr( $$field_name . '_' . $role_name ), esc_attr( $role_name ),
										esc_attr( $checked ) ); ?>
									<?php echo sprintf( '<label for="%s">%s</label>',
										esc_attr( $$field_name . '_' . $role_name ), esc_html( $role_value ) ); ?>
								</li>
							<?php endforeach; ?>
						</ul>
					</fieldset>
					<fieldset>
						<ul>
							<li>Short code: <span id="shortcodetext">
                                      <?php
                                      switch ( $$field_name ) {
	                                      case 'event_details':
		                                      $shortcode = '[eb_event_details]';
		                                      break;
	                                      case 'listing':
	                                      case 'org_details':
	                                      case 'venue_details':
		                                      $shortcode = '[eb_listview]';
		                                      break;
	                                      default:
		                                      $shortcode = '[eb_' . $$field_name . ']';
                                      }
                                      if ( $$field_name ) {
	                                      echo esc_html( $shortcode );
                                      } ?>
                                  </span>
							</li>
							<li>Page role URL: <span id="pageroleurl"><?php
									switch ( $$field_name ) {
										case 'calendar':
											$url = '/calendar/';
											break;
										case 'listing':
											$url = '/listing/';
											break;
										case 'venue_details':
											$url = '/venue/';
											break;
										case 'org_details':
											$url = '/org/';
											break;
										case 'event_details':
											$url = '/e/';
											break;
										default:
											$url = $$field_name;
									}
									echo esc_html( $url ); ?></span></li>
							<li>
								<a href="<?php echo esc_url( WP_PLUGIN_URL . '/eventbrite-events/help-window.php' ); ?>"
								   target="_blank"
								   onclick="window.open(this.href,'window','width=600,height=600,resizable,scrollbars,toolbar,menubar');return false;">Help</a>
							</li>
						</ul>
					</fieldset>
				<?php endif;
			}
		}

	}

	global $eventbrite_music;
	$eventbrite_music = new eventbrite_music();
endif;


