<?php

class Oauth {
	const TOKEN         = 'api_access_token';
	const RESPONSE_CODE = 'api_response_code';

	const ORGANIZERS_CACHE_KEY    = 'org_cache_key';
	const VENUES_CACHE_KEY        = 'venues_cache_key';
	const ORGANIZER_CACHE_TTL_MIN = '5';
	const VENUES_CACHE_TTL_MIN    = '5';

	const VENUES_ORG_KEY = 'venue_org_key';

	public function hooks() {
		add_action( 'admin_init', function () {
			if ( ! isset( $_GET[ 'code' ] ) || ! isset( $_GET[ 'page' ] ) || $_GET[ 'page' ] !== 'eb-events.php' ) {
				return;
			}
			delete_transient( self::ORGANIZERS_CACHE_KEY );
			delete_option( self::RESPONSE_CODE ); //force delete the token before setting one
			update_option( self::RESPONSE_CODE, sanitize_text_field( $_GET[ 'code' ] ) );
		} );

		add_action( 'admin_notices', [ $this, 'maybe_token_error' ] );
		add_action( 'admin_init', [ $this, 'get_access_token' ] );
	}

	public function maybe_token_error() {
		if ( get_option( self::TOKEN, false ) ) {
			return;
		}
		$class         = 'notice notice-error';
		$clean_message = sprintf(
			__(
				"You must setup your Eventbrite API authentication to use the Eventbrite Music Plugin. You can do this on the <a href='%s'>Eventbrite Settings Page.</a>",
				'eventbrite' 
			),
			admin_url( '/admin.php?page=eb-events.php' )
		);

		printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), $clean_message );
	}

	public function get_access_token() {
		if ( ! get_option( self::RESPONSE_CODE, false ) ) {
			return;
		}
		global $eventbrite_music;
		$settings = $eventbrite_music->get_options();
		$url      = 'https://www.eventbrite.com/oauth/token';
		$body     = [
			'grant_type'    => 'authorization_code',
			'client_id'     => $settings[ 'api_key' ],
			'client_secret' => $settings[ 'client_secret' ],
			'code'          => get_option( self::RESPONSE_CODE, '' ),
			'redirect_uri'  => isset( $_SERVER[ 'HTTP_HOST' ] ) ? $_SERVER[ 'HTTP_HOST' ] : '',
		];

		$request = wp_remote_post( $url, [
			'headers'   => 'Content-type: application/x-www-form-urlencoded',
			'sslverify' => false,
			'body'      => $body,
		] );

		if ( wp_remote_retrieve_response_code( $request ) !== 200 ) {
			error_log("EB Error: ". print_r($request, true));
			return '';
		}

		$response = json_decode( wp_remote_retrieve_body( $request ), true );
		$token    = isset( $response[ 'access_token' ] ) ? sanitize_text_field( $response[ 'access_token' ] ) : '';

		update_option( self::TOKEN, $token, true );
	}

	public function get_user_organizations() {
		$maybe_orgs = get_transient( self::ORGANIZERS_CACHE_KEY );
		if ( $maybe_orgs && is_array( $maybe_orgs ) ) {
			return (array) $maybe_orgs;
		}
		$url     = EVB_API_ENDPOINT . '/v3/users/me/organizations';
		$args    = [
			'sslverify' => false,
			'headers'   => [
				'Authorization' => 'Bearer ' . get_option( self::TOKEN, '' ),
			],
		];
		$request = wp_remote_get( $url, $args );
		if ( wp_remote_retrieve_response_code( $request ) !== 200 ) {
			return [];
		}

		$response = json_decode( wp_remote_retrieve_body( $request ) );
		$options  = [];
		foreach ( $response->organizations as $org ) {
			$options[ $org->id ] = $org->name;
		}
		set_transient( self::ORGANIZERS_CACHE_KEY, $options, MINUTE_IN_SECONDS * self::ORGANIZER_CACHE_TTL_MIN );

		return $options;
	}

	public function get_venues_for_organizations() {
		$maybe_venues = get_transient( self::VENUES_CACHE_KEY );
		if ( $maybe_venues && is_array( $maybe_venues ) ) {
			return (array) $maybe_venues;
		}

		$organizations = $this->get_user_organizations();
		if ( empty( $organizations ) ) {
			return [];
		}
		$args           = [
			'sslverify' => false,
			'headers'   => [
				'Authorization' => 'Bearer ' . get_option( self::TOKEN, '' ),
			],
		];
		$venues         = [];
		$org_venue_data = [];
		foreach ( $organizations as $id => $organization_name ) {
			$url = EVB_API_ENDPOINT . '/v3/organizations/%s/venues/';

			$request = wp_remote_get( sprintf( $url, $id ), $args );
			if ( wp_remote_retrieve_response_code( $request ) !== 200 ) {
				return [];
			}
			$response = json_decode( wp_remote_retrieve_body( $request ) );
			foreach ( $response->venues as $venue ) {
				$org_venue_data[ $id ][] = $venue->id;
				$venues[ $venue->id ]    = $venue->name;
			}

		}
		// Stores a key for organization and venue mapping to accurately get the organizations for a venue.
		update_option( self::VENUES_ORG_KEY, $org_venue_data );
		$venues = array_unique( $venues );
		set_transient( self::VENUES_CACHE_KEY, $venues, MINUTE_IN_SECONDS * self::VENUES_CACHE_TTL_MIN );

		return $venues;
	}

}
